@@ -1,16 +1,30 @@
 #!/usr/bin/env python
+"""
+The fMRIPrep on Docker wrapper
+
+
+This is a lightweight Python wrapper to run fMRIPrep.
+Docker must be installed and running. This can be checked
+running ::
+
+  docker info
+
+Please report any feedback to our GitHub repository
+(https://github.com/poldracklab/fmriprep) and do not
+forget to credit all the authors of software that fMRIPrep
+uses (http://fmriprep.rtfd.io/en/latest/citing.html).
+"""
 from __future__ import print_function, unicode_literals, division, absolute_import
 from builtins import int, map, input, zip
 from future import standard_library
 import sys
 import os
 import re
-import argparse
 import subprocess
 
 standard_library.install_aliases()
 
-__version__ = '0.3.2'
+__version__ = '1.0.6-1'
 __packagename__ = 'fmriprep-docker'
 __author__ = 'The CRN developers'
 __copyright__ = 'Copyright 2017, Center for Reproducible Neuroscience, Stanford University'
@@ -115,8 +129,10 @@ def check_memory(image):
     """Check total memory from within a docker container"""
     ret = subprocess.run(['docker', 'run', '--rm', '--entrypoint=free',
                           image, '-m'],
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.DEVNULL)
+                         stdout=subprocess.PIPE)
+    if ret.returncode:
+        return -1
+
     mem = [line.decode().split()[1]
            for line in ret.stdout.splitlines()
            if line.startswith(b'Mem:')][0]
@@ -147,13 +163,15 @@ def merge_help(wrapper_help, target_help):
     t_flags = sum(map(flag_re.findall, t_options), [])
 
     # The following code makes this assumption
-    assert w_flags[:2] == ['h', 'v']
+    assert w_flags[:2] == ['h', 'version']
     assert w_posargs.replace(']', '').replace('[', '') == t_posargs
 
     # Make sure we're not clobbering options we don't mean to
     overlap = set(w_flags).intersection(t_flags)
-    assert overlap == set('hvw'), "Clobbering options: {}".format(
-        ', '.join(overlap - set('hvw')))
+    expected_overlap = set(['h', 'version', 'w', 'output-grid-reference',
+                            'fs-license-file'])
+    assert overlap == expected_overlap, "Clobbering options: {}".format(
+        ', '.join(overlap - expected_overlap))
 
     sections = []
 
@@ -164,7 +182,7 @@ def merge_help(wrapper_help, target_help):
         w_options[:2],
         [opt for opt, flag in zip(t_options, t_flags) if flag not in overlap],
         w_options[2:]
-        ), [])
+    ), [])
     opt_line_length = 79 - len(start)
     length = 0
     opt_lines = [start]
@@ -191,13 +209,15 @@ def merge_help(wrapper_help, target_help):
     sections.append(w_groups[2])
 
     # All remaining sections, show target then wrapper (skipping duplicates)
-    sections.extend(t_groups[3:] + w_groups[5:])
+    sections.extend(t_groups[3:] + w_groups[6:])
     return '\n\n'.join(sections)
 
 
-def main():
+def get_parser():
+    """Defines the command line interface of the wrapper"""
+    import argparse
     parser = argparse.ArgumentParser(
-        description='fMRI Preprocessing workflow',
+        description=__doc__,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
         add_help=False)
 
@@ -209,7 +229,7 @@ def main():
 
     parser.add_argument('-h', '--help', action='store_true',
                         help="show this help message and exit")
-    parser.add_argument('-v', '--version', action='store_true',
+    parser.add_argument('--version', action='store_true',
                         help="show program's version number and exit")
 
     # Allow alternative images (semi-developer)
@@ -217,11 +237,22 @@ def main():
                         default='poldracklab/fmriprep:{}'.format(__version__),
                         help='image name')
 
+    # Options for mapping files and directories into container
+    # Update `expected_overlap` variable in merge_help() when adding to this
     g_wrap = parser.add_argument_group(
         'Wrapper options',
         'Standard options that require mapping files into the container')
     g_wrap.add_argument('-w', '--work-dir', action='store',
                         help='path where intermediate results should be stored')
+    g_wrap.add_argument('--output-grid-reference', required=False, action='store',
+                        type=os.path.abspath,
+                        help='Grid reference image for resampling BOLD files to volume template '
+                             'space.')
+    g_wrap.add_argument(
+        '--fs-license-file', metavar='PATH', type=os.path.abspath,
+        default=os.getenv('FS_LICENSE', None),
+        help='Path to FreeSurfer license key file. Get it (for free) by registering'
+             ' at https://surfer.nmr.mgh.harvard.edu/registration.html')
 
     # Developer patch/shell options
     g_dev = parser.add_argument_group(
@@ -241,11 +272,18 @@ def main():
     g_dev.add_argument('--config', metavar='PATH', action='store',
                        type=os.path.abspath, help='Use custom nipype.cfg file')
 
+    return parser
+
+
+def main():
+    """Entry point"""
+
+    parser = get_parser()
     # Capture additional arguments to pass inside container
     opts, unknown_args = parser.parse_known_args()
 
     # Set help if no directories set
-    if (opts.bids_dir, opts.output_dir) == ('', ''):
+    if (opts.bids_dir, opts.output_dir, opts.version) == ('', '', False):
         opts.help = True
 
     # Stop if no docker / docker fails to run
@@ -277,29 +315,43 @@ def main():
                 return 1
         if resp not in ('y', 'Y', ''):
             return 0
+        print('Downloading. This may take a while...')
 
     # Warn on low memory allocation
     mem_total = check_memory(opts.image)
-    if mem_total < 8000:
+    if mem_total == -1:
+        print('Could not detect memory capacity of Docker container.\n'
+              'Do you have permission to run docker?')
+        return 1
+    if not (opts.help or opts.version or '--reports-only' in unknown_args) and mem_total < 8000:
         print('Warning: <8GB of RAM is available within your Docker '
-              'environment.\nSome parts of fMRIprep may fail to complete.')
-        resp = 'N'
-        try:
-            resp = input('Continue anyway? [y/N]')
-        except KeyboardInterrupt:
-            print()
-            return 1
-        if resp not in ('y', 'Y', ''):
-            return 0
+              'environment.\nSome parts of fMRIPrep may fail to complete.')
+        if '--mem_mb' not in unknown_args:
+            resp = 'N'
+            try:
+                resp = input('Continue anyway? [y/N]')
+            except KeyboardInterrupt:
+                print()
+                return 1
+            if resp not in ('y', 'Y', ''):
+                return 0
 
     command = ['docker', 'run', '--rm', '-it']
 
     # Patch working repositories into installed package directories
     for pkg in ('fmriprep', 'niworkflows', 'nipype'):
         repo_path = getattr(opts, 'patch_' + pkg)
-        pkg_path = '{}/{}'.format(PKG_PATH, pkg)  # Always POSIX path
         if repo_path is not None:
-            command.extend(['-v', '{}:{}:ro'.format(repo_path, pkg_path)])
+            # nipype is now a submodule of niworkflows
+            if pkg == 'nipype':
+                pkg = 'niworkflows/nipype'
+            command.extend(['-v',
+                            '{}:{}/{}:ro'.format(repo_path, PKG_PATH, pkg)])
+
+    if opts.fs_license_file:
+        command.extend([
+            '-v', '{}:/opt/freesurfer/license.txt:ro'.format(
+                opts.fs_license_file)])
 
     main_args = []
     if opts.bids_dir:
@@ -318,6 +370,11 @@ def main():
         command.extend(['-v', ':'.join((opts.config,
                                         '/root/.nipype/nipype.cfg', 'ro'))])
 
+    if opts.output_grid_reference:
+        target = '/imports/' + os.path.basename(opts.output_grid_reference)
+        command.extend(['-v', ':'.join((opts.output_grid_reference, target, 'ro'))])
+        unknown_args.extend(['--output-grid-reference', target])
+
     if opts.shell:
         command.append('--entrypoint=bash')
 
@@ -332,7 +389,7 @@ def main():
         return 0
     elif opts.version:
         # Get version to be run and exit
-        command.append('-v')
+        command.append('--version')
         ret = subprocess.run(command)
         return ret.returncode
 
@@ -343,7 +400,7 @@ def main():
     print("RUNNING: " + ' '.join(command))
     ret = subprocess.run(command)
     if ret.returncode:
-        print("fmriprep: Please report errors to {}".format(__bugreports__))
+        print("fMRIPrep: Please report errors to {}".format(__bugreports__))
     return ret.returncode
 
 
